#will estimate activity using stratified approach
#input :
##areas of each stratum (from stratification map)
##proportions of each stratum (from collect earth)
##number of sample unit assigned per stratum

library(xlsx)
library(raster)
library(rgdal)
library(dplyr)

#definir le repertoire de travail
wd <- "C:/Users/User/OneDrive - alumni.manchester.ac.uk/Documents/sop1/"
raster_stratification <- "stratification_map/erpaa_stratif_ver_3_0.tif"

#definir les annees de debut et fin
date1 <- 2020
date2 <- 2021

#definir le fichier csv de sortie
csv_combined <- "output/points_combined.csv"
out_maparea <- "output/area_stratum.csv"
out_estimates <- "output/preliminary_estimates.csv"



setwd(wd)

#read in the files
#csv from operator
#reading collect earth .csv from operator
preliminary <- read.csv("output/preliminary_points_400.csv")
sample1 <- read.csv("../sop2/activity_data/Johary_collectedData_earthmadagascar_grillenational_uot_copy_1_on_280921_130723_CSV.csv")
sample2 <- read.csv("../sop2/activity_data/Baovola_collectedData_earthmadagascar_grillenational_uot_copy_1_on_011021_095527_CSV.csv")
#op3 <- read.csv("input/Topaniaina_collectedData_earthmadagascar_grillenational_uot_copy_1_on_310821_165244_CSV.csv")
#op4 <- read.csv("input/Topaniaina_collectedData_earthmadagascar_grillenational_uot_copy_1_on_310821_174330_CSV.csv")
#op5 <- read.csv("input/Baovola_collectedData_earthmadagascar_grillenational_uot_copy_1_on_240621_153855_CSV.csv")

preliminary <- preliminary[,-1]
data_csv <- rbind(preliminary,sample1,sample2)

#Check duplicated value
sum(duplicated(data_csv$id))

#check maybe there were still duplicated value with the id with ERPAA
#so let's create ID without the ERPAA_

#data_csv$UID <- gsub(("ERPAA_"),"",data_csv$id)

#sum(duplicated(data_csv$location_x,data_csv$location_y))

data_unique <- distinct(data_csv,id, .keep_all = TRUE)


write.csv(data_unique,csv_combined)


#areas of stratum
stratif_map <- raster(raster_stratification,RAT=TRUE)
#stratif_lu <- freq(stratif_map)
#stratif_lu <- as.data.frame(stratif_lu)
#stratif_lu$area_ha <- stratif_lu$count*10*10/10000
#stratif_lu <- stratif_lu[1:6,]

stratif_lu <- read.csv("stratification_map/erpaa_stratif_ver_4_0.txt")
stratif_lu <- stratif_lu[,-1] #remove OID
stratif_lu$area_ha <- stratif_lu$Count*10*10/10000
stratum_area <- sum(stratif_lu$area_ha)
stratif_lu$wh <- stratif_lu$area_ha/stratum_area

write.csv(stratif_lu,out_maparea)

#stratif_lu <- read.csv(out_maparea)


#stratif_lu <- stratif_lu[,-1]

areas <- stratif_lu$area_ha
Wi <- areas/sum(areas)

#calculer le t de student, pour la classe Foret non foret
lu_change_level1 <- as.data.frame(table(data_unique$land_use_subcategory))
t_level1 <- qt(0.95,nrow(data_unique))
lu_change_level2 <- as.data.frame(table(data_unique$land_use_subcategory))
t_level2 <- qt(0.95,nrow(data_unique))

#deforestation, on prend en compte seulement le UOT avant deforestation
# car apres c'est de la non foret et le facteur d'Emission est le meme

#ajouter les  informations sur les strates

#transforming to spatial

#setting WG84 projection
geogr <- "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs"

coordinates(data_unique) <- ~location_x+location_y
projection(data_unique) <- geogr
#getting stratum value from map
stratum_map <- extract(stratif_map,data_unique)
data_raw <- cbind.data.frame(data_unique@data,stratum_map)


#
#final_sample <- read.csv("final_sample.csv")
#data_raw <- merge(data_csv,final_sample,by.x = "id",by.y = "unique_ID", all.x = TRUE)

#recoder les valeurs pour avoir seulement foret vs non foret pour le premier stat
data_raw$land_use_subcategory <- recode(data_raw$land_use_subcategory,"FC"="FG","GO"="GG","CC"="GG","SS"="GG", "OO"="GG", "GC"="GG")

write.csv(data_raw,"output/data_with_stratum_20210928.csv")

#boucle sur les strates

sample_per_stratum <- data.frame(
  "stratum" = as.character(),
  "nh" = as.numeric()
)
deforestation_total <- data.frame("lu_level2"=as.numeric(),
                                  "fq_abs"=as.numeric(),
                                  "fq_rel" =as.numeric(),
                                  "variance"=as.numeric(),
                                  "std_error" = as.numeric(),
                                  "uncertainty" = as.numeric(),
                                  "area" = as.numeric(),
                                  "CI" =as.numeric(),
                                  "stratum" = as.numeric(),
                                  "wh" = as.numeric())


for(stratum in unique(stratif_lu$Value)){
    area_stratum <- subset(stratif_lu,stratif_lu$Value == stratum)[,3]
    wh <- subset(stratif_lu,stratif_lu$Value == stratum)[,4]
    data_defor <- data_raw %>%
      filter(stratum_map == stratum)#%>%
      nh <- nrow(data_defor)
      nb_sample <- data.frame(stratum,nh)
      sample_per_stratum <- rbind(sample_per_stratum,nb_sample)
      #filter(land_use_subcategory=="FG")
    deforestation <- data_defor %>%
      group_by(land_use_subcategory) %>%
      summarise(fq_abs = n(),
                fq_rel = fq_abs/nh
      )
    names(deforestation) <- c("lu_level2","fq_abs","fq_rel") 
    
    #variance
    deforestation$variance <- wh^2*deforestation$fq_rel*(1-deforestation$fq_rel)/nh
    #standard error
    deforestation$std_error <- sqrt(deforestation$variance)
    #uncertainties
    deforestation$uncertainty <- with(deforestation,t_level1*std_error/fq_rel)
    #area
    deforestation$area <- deforestation$fq_rel * area_stratum
    #confidence interval
    deforestation$CI <- with(deforestation,area*uncertainty)
    deforestation$stratum <- stratum
    
    deforestation_total <- rbind(deforestation_total,deforestation)
}


write.csv(deforestation_total,"output/confusion_matrix_final.csv")
write.csv(sample_per_stratum,"output/sample_per_stratum.csv")
writeOGR(data_unique,dsn = "output",layer = "activity_data_20211001",driver = "ESRI Shapefile")

#### combined uncertainties
#variances
# we need
# Wh : proportion de la strate (surface de la strate/surface total)
# Pi : proportion de l'utilisation du sol (frequence relative)
# n  : nombre d'echantillon dans la strate

## foret secondaires
fsec <- c("FSR","FSS","PLM","PLMS","PLMSf")

secondary_forest <- data_raw %>%
  filter(land_use_subdivision %in% fsec)

write.csv(degraded_forest,"secondary_forest.csv")
