# ---Clear the workspace 
rm(list=ls())

#Load libraries
library(raster)
library(rgdal)
library(stringr)
library(foreign)
library(dplyr)

###################DEFINIR DES ENTREES#####################

#set working directory. Il faut definir ici le directoire. Il faut mettre / au lieu de \
wd <- 'C:/Users/User/OneDrive - alumni.manchester.ac.uk/documents/sop1'
#le nom et le chemin vers la carte de stratification
carte_stratification <- 'stratification_map/erpaa_stratification_20210618.tif'
table_attribut_de_la_carte <- 'stratification_map/erpaa_stratification_20210618.tif.vat.dbf'
#taille de pixel de la carte
resolution_carte <- 10
#nom du fichier de sortie contenant les echantillons
echantillon_csv <- 'echantillon_aleatoire.csv'
#Set the project code
Project <- "ERPAA"
# The ISO country code. Code ISO du pays
countrycode <- 'MDG'

##################FIN PARAMETRAGE####################

setwd(wd)

# read a raster, GeoTiff or something. Il faut mettre le nom du raster de stratification. Il faut que le raster soit dans le working directory
Raster <- raster(carte_stratification,RAT=TRUE)

#Resolution de la carte utilise
Res <- resolution_carte

#File name with number of samples per stratum. Fichier avec une colonne "map_value" avec la valeur de la strate et une autre
## colonne "final" avec le nombre dechantillons

map_value <- c(11,12,22,44,55)
final <- c(100,100,100,0,100)

rp <- data.frame(map_value,final)
#rp <- read.csv('Sampling.csv', header = T)

# Fichier de sortie avec les echantillons
output_file <- echantillon_csv


##Nom du SHP avec les points qu'on va creer
Shp <- "Points_echantillonnage"

###NE TOUCHER CE QUI EST ECRIT CI DESSUS######################################################
###NE TOUCHER CE QUI EST ECRIT CI DESSUS######################################################
############################CALCUL DU NOMBRE DE PIXELS PER CLASSE DE LA CARTE##################


#t <- Raster@data@attributes # unable to read the attributes so read in the .dbf instead

t <- read.dbf(table_attribut_de_la_carte)
map_area <- as.data.frame(t)

#map_area$Ha
map_area$Ha <- map_area$Count*Res*Res/10000

#We paste the map_area in the rp
rp$map_area <- map_area$Count


##########################SELECTION DES ECHANTILLONS#########################################3

############### Generate, evaluate the least represented class (here 12) and calculate 
############### how many sample is needed to get the desired sample count

desired_sample_size <- 500
mapsize <- as.numeric(Raster@ncols)*as.numeric(Raster@nrows)
smallest_class <- min(rp$map_area)
proportion_min_map <- smallest_class/mapsize
sample_needed <- ceiling(desired_sample_size/proportion_min_map)

#now sample the raster
set.seed(200)
random_sample <- data.frame(sampleRandom(Raster,sample_needed,xy=TRUE))
#removing the nodata (128)
random_sample <- subset(random_sample,random_sample$erpaa_stratification_20210618 != 128)
names(random_sample)<- c("x_coord","y_coord","map_value")
write.csv(random_sample,"randomSample.csv")

final_sample <- data_frame("x_coord"=as.numeric(),"y_coord"=as.numeric(),"map_value"=as.numeric())
for (stratum in unique(random_sample$map_value)){
  insert_stratum <- subset(random_sample,random_sample$map_value==stratum)
  insert_stratum <- insert_stratum[1:100,] #prendre seulement 100 echantillons
  final_sample <- rbind(final_sample,insert_stratum)
}

#remove the water from this sample
final_sample <- subset(final_sample,final_sample$map_value!=44)
write.csv(final_sample,"final_sample_utm39.csv")

#ce fichier a des coordonnees UTM39
#on va reprojeter en WGS84 pour inclusion a collect earth

projUTM39 <- crs("+proj=utm +zone=39 +south +datum=WGS84 +units=m +no_defs")
geographic = crs("+proj=longlat +datum=WGS84 +no_defs")


#spatialiser les points
coordinates(final_sample) <- ~x_coord+y_coord
projection(final_sample) <- projUTM39

#reproject
final_sample_wgs84 <- spTransform(final_sample,geographic)

#add ID column
col_id <- paste(Project,1:nrow(final_sample_wgs84@data),sep = "_")
final_sample_wgs84@data$unique_ID <- col_id
final_sample_wgs84@data$ID <- paste(final_sample_wgs84@data$unique_ID,final_sample_wgs84@data$map_Value,sep = "_")

#write shapefile
writeOGR(obj = final_sample_wgs84,dsn = ".",layer = "final_sample",driver = "ESRI Shapefile", overwrite_layer = TRUE)

#write csv
write.csv(as.data.frame(final_sample_wgs84),"final_sample.csv")

