
#############################################
#Classification d'image sentinel pour
#la creation d'UOT en vue de la stratification
# et planification de l'echantillonage
#
# contacts : LOFM, 034343434 e-mail
#         
############################################


# importer les modules

library(randomForest)
library(raster)
library (rgdal)
library(xlsx)
library(spatialEco)


#definition du repertoire de travail
setwd("F:/Monitoring_2021")

#lire shapefiles contenant les zones AOI zones d'entrainement
#
#readOGR(dsn = lien_vers_le_fichier, layer = nom_du_fichiers_sans_extension)


#### assurez que les shp et les rasters ont
#### les memes CRS
roi_forest <- readOGR(dsn = "./39LVD/00_ROIs/ROI_v3",layer="FF_v3")
roi_nonfor<- readOGR(dsn = "./39LVD/00_ROIs/ROI_v3",layer="NN_v3")
roi_defor <- readOGR(dsn = "./39LVD/00_ROIs/ROI_v3",layer="FN_v3")
roi_water <- readOGR(dsn = "./39LVD/00_ROIs/ROI_v3",layer="WW_v3")

#verifier les fichiers importes
#plot(roi_forest)

roi_all <- bind(roi_forest,roi_nonfor,roi_defor,roi_water)

#plot(roi_all)

#lire une image raster, date1 et date2
s2_image2 <- brick("39LVD/39LVD_oct2021_avr2022_32739.tif")
s2_image1 <- brick("39LVD/39LVD_dec2020_mars2021_32739.tif")

#plot(s2_image1)
# renommer les bandes selon l'image qui a ete importee
s2_bandnames <- c('B2', 'B3', 'B4','B5','B6','B7','B8','B8A','B11','B12','NDVI')
names(s2_image1) <- s2_bandnames
names(s2_image2) <- s2_bandnames

#combiner les deux dates en une seule image
inImage <- stack(s2_image1,s2_image2)

#plot(inImage)

#creation des point dans les polygones
set.seed(4)
sample_ff <- spsample(roi_forest,4000,"random")
to_train <- sample(1:4000,2000)
to_valid <- (1:4000)[-to_train]
training_ff <- sample_ff[to_train]
valid_ff <- sample_ff[to_valid]

sample_nn <- spsample(roi_nonfor,4000,"random")
training_nn <- sample_nn[to_train]
valid_nn <- sample_nn[to_valid]

sample_fn <- spsample(roi_defor,4000,"random")
training_fn <- sample_fn[to_train]
valid_fn <- sample_fn[to_valid]

sample_ww <- spsample(roi_water,4000,"random")
training_ww <- sample_ww[to_train]
valid_ww <- sample_ww[to_valid]


###
###
# separation des points d'entrainement et de validation

#on peut combiner les points maintenant
point_sample <- bind(training_ff,training_nn,training_fn,training_ww)


#re-obtenir les attributs
point_sample <- point.in.poly(point_sample,roi_all)

#attacher les attributs de l'image (valeur spectrales)
#sur les points d'entrainement
training_point <- extract(inImage,point_sample)
writeOGR(point_sample,'./39LVD/00_ROIs/ROI_v3','training_point_LVD','ESRI Shapefile')


# creation du modele randomforest
rf <- randomForest(x=training_point,y=as.factor(point_sample@data$class),ntree = 501,importance = TRUE, progress="window")

#graphic de rf
plot(rf)

#####################################################
# Creation de la carte de changement, changez le nom
#####################################################

predict(inImage, rf, filename="39LVD/01_Classification/39LVD_classif_.img", type="response",index=1, na.rm=TRUE, progress="window", overwrite=TRUE)


#validation ############
########################
point_valid <- bind(valid_ff,valid_fn,valid_nn,valid_ww)
#ajouter les attributs
point_valid <- point.in.poly(point_valid,roi_all)#
#re-importer l'image classifie
classified <- brick("39LVD/01_Classification/39LVD_classif_.img")
#attaching raster attributes to shapefile points
validation <- extract(classified,point_valid)
point_valid <- data.frame(point_valid,validation)
write.csv(table(point_valid$class,point_valid$Layer_1),"./39LVD/confusion_matrix.csv")
confusion_matrix <- table(point_valid$class,point_valid$Layer_1)
#omission error
ff_omission <-
  (confusion_matrix[2,1]+confusion_matrix[3,1]+confusion_matrix[4,1])/500*100
fn_omission <-
  (confusion_matrix[1,2]+confusion_matrix[3,2]+confusion_matrix[4,2])/500*100
nn_omission <-
  (confusion_matrix[1,3]+confusion_matrix[2,3]+confusion_matrix[4,3])/500*100
nn_omission <-
  (confusion_matrix[1,3]+confusion_matrix[2,3]+confusion_matrix[4,3])/500*100
ww_omission <-
  (confusion_matrix[1,4]+confusion_matrix[2,4]+confusion_matrix[3,4])/500*100
omission_error <-c(ff_omission,fn_omission,nn_omission,ww_omission)

#comission error
ff_comission <-
  (confusion_matrix[1,2]+confusion_matrix[1,3]+confusion_matrix[1,4])/500*100
fn_comission <-
  (confusion_matrix[2,1]+confusion_matrix[2,3]+confusion_matrix[2,4])/500*100
nn_comission <-
  (confusion_matrix[3,1]+confusion_matrix[3,2]+confusion_matrix[3,4])/500*100
ww_comission <-
  (confusion_matrix[4,1]+confusion_matrix[4,2]+confusion_matrix[4,3])/500*100
comission_error <-c(ff_comission,fn_comission,nn_comission,ww_comission)
erreur <- data.frame(c('ff','fn','nn','ww'),omission_error,comission_error)
names(erreur) <- c('class','omission','comission')
#ecrire les resultats en excel
write.xlsx(erreur,"./39LVD/erreur.xlsx")
